#include "DropshellScriptManager.hpp"
#include <fstream>
#include <string>
#include <vector>
#include <filesystem>
#include <algorithm>
#include <iostream>

namespace dropshelltool {
    extern const std::filesystem::path DROPSHELL_RC_PATH;
}

DropshellScriptManager::DropshellScriptManager() {}

// Helper function to trim whitespace from both ends of a string
static std::string trim(const std::string& s) {
    auto start = s.begin();
    while (start != s.end() && std::isspace(*start)) ++start;
    auto end = s.end();
    do { --end; } while (std::distance(start, end) > 0 && std::isspace(*end));
    return std::string(start, end + 1);
}

void DropshellScriptManager::ensureExists() const {
    if (!std::filesystem::exists(dropshelltool::DROPSHELL_RC_PATH)) {
        std::filesystem::create_directories(dropshelltool::DROPSHELL_RC_PATH.parent_path());
        std::ofstream outfile(dropshelltool::DROPSHELL_RC_PATH, std::ios::trunc);
        outfile << "# This file is managed by getpkg. Do not edit manually!" << std::endl;
        outfile.close();
    }
}

void DropshellScriptManager::addToolEntry(const std::string& toolName, const std::string& toolDir) {
    ensureExists();
    std::ifstream infile(dropshelltool::DROPSHELL_RC_PATH);
    std::vector<std::string> lines;
    std::string line;
    std::string exportLine = "export PATH=\"" + toolDir + ":$PATH\" # getpkg:" + toolName;
    bool found = false;
    while (std::getline(infile, line)) {
        if (line.find("# getpkg:" + toolName) != std::string::npos) {
            found = true;
            lines.push_back(exportLine);
        } else {
            lines.push_back(line);
        }
    }
    infile.close();
    if (!found) lines.push_back(exportLine);
    std::ofstream outfile(dropshelltool::DROPSHELL_RC_PATH, std::ios::trunc);
    for (const auto& l : lines) outfile << l << "\n";
    outfile.close();
}

void DropshellScriptManager::removeToolEntry(const std::string& toolName) {
    ensureExists();
    std::ifstream infile(dropshelltool::DROPSHELL_RC_PATH);
    std::vector<std::string> lines;
    std::string line;
    while (std::getline(infile, line)) {
        if (line.find("# getpkg:" + toolName) == std::string::npos) {
            lines.push_back(line);
        }
    }
    infile.close();
    std::ofstream outfile(dropshelltool::DROPSHELL_RC_PATH, std::ios::trunc);
    for (const auto& l : lines) outfile << l << "\n";
    outfile.close();
}

void DropshellScriptManager::addAutocomplete(const std::string& toolName) {
    ensureExists();
    std::ifstream infile(dropshelltool::DROPSHELL_RC_PATH);
    std::vector<std::string> lines;
    std::string line;

    std::string funcName = "_" + toolName + "_autocomplete";
    
    std::string blockStart = "# GETPKG-AUTOCOMPLETE-START: " + toolName;
    std::string blockEnd = "# GETPKG-AUTOCOMPLETE-END: " + toolName;
    
    std::vector<std::string> funcLines;
    funcLines.push_back(blockStart);
    funcLines.push_back(funcName + "() {");
    funcLines.push_back("    local cur=\"${COMP_WORDS[COMP_CWORD]}\"");
    funcLines.push_back("    mapfile -t completions < <(" + toolName + " autocomplete \"${COMP_WORDS[@]:1:${COMP_CWORD}-1}\")");
    funcLines.push_back("    mapfile -t COMPREPLY < <(compgen -W \"${completions[*]}\" -- \"$cur\")");
    funcLines.push_back("}");
    funcLines.push_back(blockEnd);

    std::string completeLine = "complete -F " + funcName + " " + toolName + " # autocomplete: " + toolName;
    bool blockFound = false;
    bool inBlock = false;
    while (std::getline(infile, line)) {
        if (!inBlock && trim(line) == blockStart) {
            blockFound = true;
            inBlock = true;
            continue;
        }
        if (inBlock) {
            if (trim(line) == blockEnd) {
                inBlock = false;
                // Add all function lines
                for (const auto& funcLine : funcLines) {
                    lines.push_back(funcLine);
                }
            }
            continue; // Skip all lines in the old block
        }
        else
            lines.push_back(line);
    }
    infile.close();
    if (!blockFound) {
        // Add all function lines
        for (const auto& funcLine : funcLines) {
            lines.push_back(funcLine);
        }
    }
    std::ofstream outfile(dropshelltool::DROPSHELL_RC_PATH, std::ios::trunc);
    for (const auto& l : lines) outfile << l << "\n";
    outfile.close();
}

void DropshellScriptManager::removeAutocomplete(const std::string& toolName) {
    ensureExists();
    std::ifstream infile(dropshelltool::DROPSHELL_RC_PATH);
    std::vector<std::string> lines;
    std::string line;
    
    std::string blockStart = "# GETPKG-AUTOCOMPLETE-START: " + toolName;
    std::string blockEnd = "# GETPKG-AUTOCOMPLETE-END: " + toolName;
    bool inBlock = false;
    
    while (std::getline(infile, line)) {
        if (trim(line) == blockStart) {
            inBlock = true;
            continue;
        }
        if (trim(line) == blockEnd) {
            inBlock = false;
            continue;
        }
        if (inBlock) {
            continue;
        }
        // Also skip the complete line
        if (line.find("# autocomplete: " + toolName) != std::string::npos) {
            continue;
        }
        lines.push_back(line);
    }
    infile.close();
    
    std::ofstream outfile(dropshelltool::DROPSHELL_RC_PATH, std::ios::trunc);
    for (const auto& l : lines) outfile << l << "\n";
    outfile.close();
}